@@ -1,226 +1,42 @@
 # -*- coding: utf-8 -*-
 from collections import deque
+from queue import queue
+from stack import stack
 
 import numpy
-#from scipy import linalg
-#from scipy import dot
-#import scipy
-
 
 ## Original: http://www.ece.arizona.edu/~denny/python_nest/graph_lib_1.0.1.html
 ## Modifications: Alex Toney
-class Queue:
-    def __init__(self):
-        self.queue = deque()
-
-    def empty(self):
-        if(len(self.queue) > 0):
-            return False
-        else:
-            return True
-
-    def count(self):
-        return len(self.queue)
-
-    def add(self, item):
-        self.queue.append(item)	
-
-    def remove(self):
-        item = self.queue[0]
-        self.queue = self.queue[1:]
-        return item
-
-class Stack:
-    def __init__(self):
-        self.s=[]
-
-    def empty(self):
-        if(len(self.s) > 0):
-            return 0
-        else:
-            return 1
-
-    def count(self):
-        return len(self.s)
-
-    def push(self, item):
-        ts=[item]
-        for i in self.s:
-            ts.append(i)
-        self.s=ts
-
-    def pop(self):
-        item=self.s[0]
-        self.s=self.s[1:]
-        return item
-
-
-class GraphException(Exception):
+class graphException(Exception):
     def __init__(self, value):
         self.value = value
     def __str__(self):
         return repr(self.value)
 
-class graph:
-    """ A graph with edges, nodes, and edge types """
+class graph_container:
+    next_edge_id = None
+    next_node_id = None
+    nodes        = None
+    edges        = None
+    hidden_edges = None
+    hidden_nodes = None
+    edge_types   = None
+    types        = None
+    node_number  = None
+    
     def __init__(self):
         self.next_edge_id = 0
         self.next_node_id = 0
-        self.nodes = {}
-        self.edges = {}
+        self.nodes        = {}
+        self.edges        = {}
         self.hidden_edges = {}
         self.hidden_nodes = {}
-        self.edge_types = []
-        self.types = {}
-        self.node_number = {}
-
-    
-    def copy(self, G):
-        """ Performs a copy of the graph, G, into self.
-        hidden edges and hidden nodes are not copied.
-        node_id's remain consistent across self and G, 
-        however edge_id's do not remain consistent.
-        Need to implement copy operator on node_data
-        and edge data.
-
-        :param G: Hypergraph instance
-
-        """
-        
-        self.nodes = {}
-        self.edges = {}
-        self.hidden_edges = {}
-        self.hidden_nodes = {}
-        self.next_edge_id = 0
-        
-        ## Copy nodes.
-        G_node_list = G.node_list()
-        for G_node in G_node_list:
-            self.add_node(G_node,G.node_data(G_node))
-            
-        ## Copy edges.
-        for G_node in G_node_list:
-            out_edges = G.out_arcs(G_node)
-            for edge in out_edges:
-                tail_id=G.tail(edge)
-                self.add_edge(G_node, tail_id, G.edge_data(edge))
+        self.edge_types   = []
+        self.types        = {}
+        self.node_number  = {}
 
 
-    def add_node(self, node_id, node_data=None, ignore_dupes=True):
-        """ Creates a new node with id node_id.  Arbitrary data can be attached
-        to the node via the node_data parameter.
-
-        :param node_id: the node's id can be any hashable type
-        :param node_data: associated data
-
-        """
-        if (not self.nodes.has_key(node_id)) and (not self.hidden_nodes.has_key(node_id)):
-            self.nodes[node_id]=([],[],node_data, self.next_node_id)
-            self.node_number[self.next_node_id] = node_id
-            
-            self.next_node_id += 1
-        else:
-            if not ignore_dupes:
-                raise GraphException('Duplicate Node: %s', node_id)
-
-
-    
-    def delete_node(self, node_id):
-        """ Deletes the node and all in and out arcs. """
-        
-        #--Remove fanin connections.
-        in_edges=self.in_arcs(node_id)
-        for edge in in_edges:
-            self.delete_edge(edge)
-            
-        #--Remove fanout connections.
-        out_edges = self.out_arcs(node_id)
-        for edge in out_edges:
-            self.delete_edge(edge)
-            
-        #--Delete node.
-        del self.nodes[node_id]
-
-
-    
-    def delete_edge(self, edge_id):
-        """ Deletes the edge. """
-        head_id = self.head(edge_id)
-        tail_id = self.tail(edge_id)
-        head_data = map(None, self.nodes[head_id])
-        tail_data = map(None, self.nodes[tail_id])
-        head_data[1].remove(edge_id)
-        tail_data[0].remove(edge_id)
-        del self.edges[edge_id]
-        
-    def delete_edge_by_type(self, edge_type):
-        if edge_type in self.types:
-            del self.types[edge_type]
-            
-    def with_merge(self, head_id, tail_id, edge_data, edge_type):
-         edge = self.edge(head_id, tail_id)
-         if edge != False:
-             cEdge_type = self.edges[edge][3]
-             if edge_type != edge_type:
-                 return
-             
-             current_edge_data = self.edges[edge][2]
-             if isinstance(current_edge_data, list):
-                 if isinstance(edge_data, list):
-                     for item in edge_data:
-                         current_edge_data.append(item)
-                 else:
-                     current_edge_data.append(item)
-             else:
-                 raise GraphException('ListError: no list found')
-             
-             return edge
-
-         else:
-             return False
-                     
-                         
-    def add_edge(self, head_id, tail_id, edge_data=None, head_data=None,
-                 tail_data=None, edge_type=None, with_merge=False):
-        """ adds an edge (head_id, tail_id).
-        arbitrary data can be attached to the edge via edge_data
-        with_merge will merge the edge with another of of the same nodes and only add the data"""
-        
-        edge_id = self.next_edge_id
-        self.next_edge_id = self.next_edge_id + 1
-        
-        if with_merge and edge_data:
-           merged = self.with_merge(head_id, tail_id, edge_data, edge_type)
-           if merged != False:
-               return merged
-
-        if edge_type not in self.edge_types:
-            self.edge_types.append(edge_type)    
-            self.types[edge_type] = []
-
-        self.types[edge_type].append((head_id, edge_data, tail_id))
-            
-            
-        self.edges[edge_id] = [head_id, tail_id, edge_data, edge_type]
-
-        
-        if not self.nodes.has_key(head_id):
-            self.add_node(head_id, node_data=head_data)
-            
-        if not self.nodes.has_key(tail_id):
-            self.add_node(tail_id, node_data=tail_data)
-        
-        try:
-            mapped_head_data = map(None, self.nodes[head_id])
-            mapped_head_data[1].append(edge_id)
-            mapped_tail_data = map(None, self.nodes[tail_id])
-            mapped_tail_data[0].append(edge_id)
-        except Exception, E:
-            raise GraphException('Exception %s nodes: %s' % (E, self.nodes))
-        
-        return edge_id
-
-    
+class modifier_mixin:
     def hide_edge(self, edge_id):
         """ Removes the edge from the normal graph, but does not delete
         its information.  The edge is held in a separate structure
@@ -287,8 +103,165 @@ class graph:
         for node in hidden_node_list:
             self.nodes[node]=self.hidden_nodes[node]
             del self.hidden_nodes[node]
+            
+    def _with_merge(self, head_id, tail_id, edge_data, edge_type):
+         edge = self.edge(head_id, tail_id)
+         if edge != False:
+             cEdge_type = self.edges[edge][3]
+             if edge_type != edge_type:
+                 return
+             
+             current_edge_data = self.edges[edge][2]
+             if isinstance(current_edge_data, list):
+                 if isinstance(edge_data, list):
+                     for item in edge_data:
+                         current_edge_data.append(item)
+                 else:
+                     current_edge_data.append(item)
+             else:
+                 raise GraphException('ListError: no list found')
+             
+             return edge
 
-    #--Returns 1 if the node_id is in the graph and 0 otherwise.
+         else:
+             return False
+    def copy(self, G):
+        """ Performs a copy of the graph, G, into self.
+        hidden edges and hidden nodes are not copied.
+        node_id's remain consistent across self and G, 
+        however edge_id's do not remain consistent.
+        Need to implement copy operator on node_data
+        and edge data.
+
+        :param G: Hypergraph instance
+
+        """
+        
+        self.nodes = {}
+        self.edges = {}
+        self.hidden_edges = {}
+        self.hidden_nodes = {}
+        self.next_edge_id = 0
+        
+        ## Copy nodes.
+        G_node_list = G.node_list()
+        for G_node in G_node_list:
+            self.add_node(G_node,G.node_data(G_node))
+            
+        ## Copy edges.
+        for G_node in G_node_list:
+            out_edges = G.out_arcs(G_node)
+            for edge in out_edges:
+                tail_id=G.tail(edge)
+                self.add_edge(G_node, tail_id, G.edge_data(edge))
+
+
+    
+
+class create_mixin:
+    def new_node(self, node_id, node_data=None, ignore_dupes=True):
+        """ Creates a new node with id node_id.  Arbitrary data can be attached
+        to the node via the node_data parameter.
+
+        :param node_id: the node's id can be any hashable type
+        :param node_data: associated data
+
+        """
+        if (not self.nodes.has_key(node_id)) and (not self.hidden_nodes.has_key(node_id)):
+            self.nodes[node_id]=([],[],node_data, self.next_node_id)
+            self.node_number[self.next_node_id] = node_id
+            
+            self.next_node_id += 1
+        else:
+            if not ignore_dupes:
+                raise GraphException('Duplicate Node: %s', node_id)
+
+    def new_edge(self, head_id, tail_id, edge_data=None, head_data=None,
+                 tail_data=None, edge_type=None, with_merge=False):
+        """ adds an edge (head_id, tail_id).
+        arbitrary data can be attached to the edge via edge_data
+        with_merge will merge the edge with another of of the same nodes and only add the data"""
+        
+        edge_id = self.next_edge_id
+        self.next_edge_id = self.next_edge_id + 1
+        
+        if with_merge and edge_data:
+           merged = self._with_merge(head_id, tail_id, edge_data, edge_type)
+           if merged != False:
+               return merged
+
+        if edge_type not in self.edge_types:
+            self.edge_types.append(edge_type)    
+            self.types[edge_type] = []
+
+        self.types[edge_type].append((head_id, edge_data, tail_id))
+            
+            
+        self.edges[edge_id] = [head_id, tail_id, edge_data, edge_type]
+
+        
+        if not self.nodes.has_key(head_id):
+            self.add_node(head_id, node_data=head_data)
+            
+        if not self.nodes.has_key(tail_id):
+            self.add_node(tail_id, node_data=tail_data)
+        
+        try:
+            mapped_head_data = map(None, self.nodes[head_id])
+            mapped_head_data[1].append(edge_id)
+            mapped_tail_data = map(None, self.nodes[tail_id])
+            mapped_tail_data[0].append(edge_id)
+        except Exception, E:
+            raise GraphException('Exception %s nodes: %s' % (E, self.nodes))
+        
+        return edge_id
+    def new_link(self):
+        pass
+    def new_type(self):
+        pass
+        
+class delete_mixin:
+    def delete_node(self, node_id):
+        """ Deletes the node and all in and out arcs. """
+        
+        ## Remove fanin connections.
+        in_edges=self.in_arcs(node_id)
+        for edge in in_edges:
+            self.delete_edge(edge)
+            
+        ## Remove fanout connections.
+        out_edges = self.out_arcs(node_id)
+        for edge in out_edges:
+            self.delete_edge(edge)
+            
+        ## Delete node.
+        del self.nodes[node_id]
+
+
+    def delete_edge(self, edge_id):
+        """ Deletes the edge. """
+        head_id = self.head(edge_id)
+        tail_id = self.tail(edge_id)
+        head_data = map(None, self.nodes[head_id])
+        tail_data = map(None, self.nodes[tail_id])
+        head_data[1].remove(edge_id)
+        tail_data[0].remove(edge_id)
+        del self.edges[edge_id]
+        
+    def delete_edge_by_type(self, edge_type):
+        if edge_type in self.types:
+            del self.types[edge_type] 
+            
+class graph(graph_container, modifier_mixin, delete_mixin, create_mixin):
+    """ A graph with edges, nodes, and edge types """
+    
+    def __init__(self):
+        graph_container.__init__(self)
+        modifier_mixin.__init__(self)
+        delete_mixin.__init__(self)   
+        create_mixin.__init__(self)
+
+    ## Returns 1 if the node_id is in the graph and 0 otherwise.
     def has_node(self, node_id):
         if self.nodes.has_key(node_id):
             return True
@@ -669,4 +642,82 @@ class graph:
                     incidMatrix[arc, vertex] += 1
                 
         return incidMatrix
+
+    def incidenceList(self):
+        wholeList = []
+        ## |V| = nodeCount
+        nodeCount = self.number_of_nodes()+1
+        ## |E| = edgeCount
+        edgeCount = self.number_of_edges()+1
+        allNodes = self.node_list()
+        
+        for node in allNodes:
+            vertex = self.node(node)[3]
+            for arc in self.out_arcs(node):
+                wholeList.append([arc, vertex])
+                
+        return wholeList
+
+
     
+    def to_dot_file(self, primary_type=None, exclude_filter=None):
+        import pydot
+        import time
+        import cPickle
+        import hashlib
+        import os
+        
+        self_hash = hashlib.sha224(cPickle.dumps(self)).hexdigest()
+
+        unique_colors = ['skyblue1','maroon1','yellowgreen','olivedrab3','red3','royalblue3',]
+        unique_color_dict = {}
+        
+        callgraph = pydot.Dot(graph_type='digraph', fontname="Verdana", fontsize="9")
+        node_list = self.nodes.keys()
+
+        for types in self.edge_types:
+            if types != None:
+                color = unique_colors.pop()
+                unique_color_dict[types] = color
+            
+        for node in node_list:
+            for edge in self.node_out_edges(node):
+                edge_type = edge[4]
+                
+                if unique_color_dict.has_key(edge_type):
+                    color = unique_color_dict[edge_type]
+                else:
+                    color = 'black'
+                
+                if len(edge[1]) == 1:
+                    label = str(edge[1][0])
+                else:
+                    label = ','.join(map(str, edge[1]))
+                        
+                if primary_type != None and edge_type == primary_type:
+                    head = pydot.Node(edge[2], shape='doublecircle')
+                    tail = pydot.Node(edge[3], shape='doublecircle')
+                    callgraph.add_node(head)
+                    callgraph.add_node(tail)
+                        
+                    current_edge = pydot.Edge(edge[2], edge[3], label=label, color=color, weight="3", fontsize="8")
+                else:
+                    current_edge = pydot.Edge(edge[2], edge[3], label=label, color=color, fontsize="8")
+
+                callgraph.add_edge(current_edge)
+
+        if primary_type and primary_type in self.types:
+            heads = []
+            for x in self.types[primary_type]:
+                heads.append(x[0])
+            sentence = '-'.join(heads)
+            file_name = '%s-%s' % (sentence, self_hash)
+        else:
+            file_name = self_hash
+            
+        file = os.path.join('data/graphs', '%s-map.svg' % file_name.strip().replace('/','')[:100])
+        if not os.path.exists(file):
+            callgraph.write_svg(file)
+
+        print 'Graph: %s' % (file)
+
